#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.gallery.interfaces import IGalleryParagraph, IBaseGallery, IGalleryRenderer
from pyams_content.component.gallery.zmi.interfaces import IGalleryImagesView
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphSummary
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor, IParagraphContainerView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons, IInnerSubForm
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu, IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.gallery.paragraph import Gallery
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu, BaseParagraphPropertiesEditForm
from pyams_content.shared.common.zmi import WfSharedContentPermissionMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config, BaseContentProvider
from pyams_zmi.form import AdminDialogAddForm, InnerAdminDisplayForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer, Interface

from pyams_content import _


@viewlet_config(name='add-gallery.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=65)
class GalleryAddMenu(BaseParagraphAddMenu):
    """Gallery add menu"""

    label = _("Add images gallery...")
    label_css_class = 'fa fa-fw fa-picture-o'
    url = 'add-gallery.html'
    paragraph_type = 'Gallery'


@pagelet_config(name='add-gallery.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class GalleryAddForm(AdminDialogAddForm):
    """Gallery add form"""

    legend = _("Add new gallery")
    icon_css_class = 'fa fa-fw fa-picture-o'

    fields = field.Fields(IGalleryParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-gallery.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(GalleryAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'

    def create(self, data):
        return Gallery()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-gallery.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class GalleryAJAXAddForm(BaseParagraphAJAXAddForm, GalleryAddForm):
    """Gallery paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IGalleryParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class GalleryPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Gallery properties edit form"""

    legend = _("Edit gallery properties")
    icon_css_class = 'fa fa-fw fa-picture-o'

    fields = field.Fields(IGalleryParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(GalleryPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'


@view_config(name='properties.json', context=IGalleryParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class GalleryPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, GalleryPropertiesEditForm):
    """Gallery paragraph properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        updated = changes.get(IBaseGallery, ())
        if 'title' in updated:
            return {'status': 'success',
                    'events': [{
                        'event': 'myams.refresh',
                        'options': {
                            'handler': 'PyAMS_content.paragraphs.refreshParagraph',
                            'object_name': self.context.__name__,
                            'title': II18n(self.context).query_attribute('title', request=self.request),
                            'visible': self.context.visible
                        }
                    }]}
        else:
            return super(GalleryPropertiesAJAXEditForm, self).get_ajax_output(changes)


@adapter_config(context=(IGalleryParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm)
class GalleryInnerEditForm(GalleryPropertiesEditForm):
    """Gallery properties inner edit form"""

    legend = None

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


#
# Gallery contents view
#

@adapter_config(name='gallery-images', context=(IGalleryParagraph, IPyAMSLayer, GalleryInnerEditForm),
                provides=IInnerSubForm)
@template_config(template='templates/gallery-images.pt', layer=IPyAMSLayer)
@implementer(IGalleryImagesView)
class GalleryContentsView(WfSharedContentPermissionMixin, InnerAdminDisplayForm):
    """Gallery contents edit form"""

    fields = field.Fields(Interface)
    weight = 10

    def get_title(self, image):
        return II18n(image).query_attribute('title', request=self.request)


@viewlet_config(name='add-image.menu', context=IGalleryParagraph, view=GalleryContentsView,
                manager=IWidgetTitleViewletManager)
class GalleryImageAddMenu(WfSharedContentPermissionMixin, ToolbarAction):
    """Gallery image add menu"""

    label = _("Add image(s)")

    url = 'add-image.html'
    modal_target = True
    stop_propagation = True


#
# Gallery paragraph summary
#

@adapter_config(context=(IGalleryParagraph, IPyAMSLayer), provides=IParagraphSummary)
class GalleryParagraphSummary(BaseContentProvider):
    """Gallery summary"""

    def __init__(self, context, request):
        super(GalleryParagraphSummary, self).__init__(context, request)
        self.renderer = request.registry.queryMultiAdapter((context, request), IGalleryRenderer,
                                                           name=self.context.renderer)

    language = None

    def update(self):
        if self.renderer is not None:
            self.renderer.language = self.language
            self.renderer.update()

    def render(self):
        if self.renderer is not None:
            return self.renderer.render()
        else:
            return ''
